/*
  buffer.hpp

  Copyright (C) 2012 Taiji Yamada <taiji@aihara.co.jp>
 */
#ifndef _BUFFER_HPP_
#define _BUFFER_HPP_
#include <cstdlib>

struct buffer {
  size_t size;
  char *ptr;
  buffer() : size(0), ptr(NULL) {}
  virtual ~buffer() { clear(); }
  char *append(const char *p, const size_t s)
  {
    if (!s) return ptr;
    if (!(ptr = ptr ? (char *)realloc(ptr, size + s) : (char *)malloc(s)))
      return NULL;
    memcpy(&ptr[size], p, s);
    size += s;
    return ptr;
  };
  char *text_append(const char *text, const size_t size)
  {
    char *p, *q;
    for (q=p=(char *)text; p!=(char *)text+size; ++p)
      if (*p == char(0)) {
        append(q, p-q);
        q = p+1;
      }
    append(q, p-q);
    return ptr;
  }
  void clear()
  {
    if (ptr) { free(ptr); ptr = NULL; }
    size = 0;
  }
};

#endif
