#ifndef _GEOMETRY_HPP_
#define _GEOMETRY_HPP_

template <typename T>
struct point {
  T x, y;
  point(T x_ = 0, T y_ = 0) : x(x_), y(y_) {}
  template <typename C>
  point(const point<C> &p) : x(p.x), y(p.y) {}
  bool operator==(const point &p) const
  {
    return x == p.x && y == p.y;
  }
  bool operator!=(const point &p) const { return !(*this == p); }
  const point operator+(const point &p) const
  {
    point q(x + p.x, y + p.y);
    return q;
  }
  point &operator+=(const point &p) { *this = *this + p; return *this; }
  const point operator-(const point &p) const
  {
    point q(x - p.x, y - p.y);
    return q;
  }
  point &operator-=(const point &p) { *this = *this - p; return *this; }
  template <typename C>
  const point operator*(const C &c) const
  {
    point q(x*c, y*c);
    return q;
  }
  template <typename C>
  point<T> &operator*=(const C &c) { *this = *this * c; return *this; }
  template <typename C>
  const point<T> operator/(const C &c) const
  {
    point<T> q(x/c, y/c);
    return q;
  }
  template <typename C>
  point<T> &operator/=(const C &c) { *this = *this / c; return *this; }
  template <typename C>
  const point<T> operator*(const point<C> &p) const
  {
    point<T> q(x*p.x, y*p.y);
    return q;
  }
  template <typename C>
  point<T> &operator*=(const point<C> &p) { *this = *this * p; return *this; }
  template <typename C>
  const point<T> operator/(const point<C> &p) const
  {
    point<T> q(x/p.x, y/p.y);
    return q;
  }
  template <typename C>
  point &operator/=(const point<C> &p) { *this = *this / p; return *this; }
};

template <typename T>
struct dimension {
  T w, h;
  dimension(T w_ = 0, T h_ = 0) : w(w_), h(h_) {}
  template <typename C>
  dimension(const dimension<C> &d) : w(d.w), h(d.h) {}
};

template <typename T, typename C = T>
struct rectangle {
  point<T> p;
  dimension<C> d;
  rectangle(T x = 0, T y = 0, C w = 0, C h = 0) :
    p(x, y), d(w, h) {}
  rectangle(const point<T> &p_, const point<C> &d_) :
    p(p_), d(d_) {}
  bool is_inner(const point<T> &q)
  {
    return (p.x <= q.x && q.x <= p.x + d.w - 1 && p.y <= q.y && q.y + d.h - 1);
  }
  const rectangle operator+(const rectangle &r)
  {
    point<T> a(std::min(p.x, r.p.x), std::min(p.y, r.p.y)), b(std::max(p.x+d.w, r.p.x+r.d.w)-1, std::max(p.y+d.h, r.p.y+r.d.h)-1);
    return rectangle<T, C>(a.x, a.y, b.x-a.x+1, b.y-a.y+1);
  }
  rectangle &operator+=(const rectangle &r) { *this = *this + r; return *this; }
};

#endif
