/*
  text_list.h

  Copyright (C) 2012 Taiji Yamada <taiji@aihara.co.jp>
 */
#ifndef _TEXT_LIST_H_
#define _TEXT_LIST_H_
#include <stdlib.h>

inline
void text_list_free(char **text_list, const size_t count)
{
#if defined(INEFFICIENT)
  size_t i;
  for (i=0; i!=count; ++i)
    free(text_list[i]);
  free(text_list);
#else
  (void)count;
  free(text_list[0]);
  free(text_list);
#endif
}
inline
char **text_list_alloc(const char *ptr, const size_t size, size_t *count_return)
{
#if defined(INEFFICIENT)
  char **text_list, *p, *q;
  size_t c = 0, i = 0, l;
  for (p=(char *)ptr; p!=(char *)ptr+size; ++p)
    if (*p == '\n')
      ++c;
  if (p != q) ++c;
  if (count_return) *count_return = c;
  if (!(text_list = (char **)malloc(sizeof(char *)*(c+1)))) return NULL;
  for (q=p=(char *)ptr; p!=(char *)ptr+size; ++p)
    if (*p == '\n') {
      l = p - q + 1;
      if (!(text_list[i] = (char *)malloc(sizeof(char)*(l+1)))) {
        text_list_free(text_list, i);
        return NULL;
      }
      memset(text_list[i], 0, sizeof(char)*(l+1));
      memcpy(text_list[i], q, sizeof(char)*l);
      ++i;
      q = p+1;
    }
  if (p != q) {
    size_t l = p - q;
    if (!(text_list[i] = (char *)malloc(sizeof(char)*(l+1)))) {
      text_list_free(text_list, i);
      return NULL;
    }
    memset(text_list[i], 0, sizeof(char)*(l+1));
    memcpy(text_list[i], q, sizeof(char)*l);
    ++i;
  }
  text_list[i] = NULL;
  return text_list;
#else
  char **text_list, *p, *q;
  size_t c = 0, s = 0, i = 0, l;
  for (q=p=(char *)ptr; p!=(char *)ptr+size; ++p)
    if (*p == '\n') {
      l = p - q + 1;
      s += l + 1;
      ++c;
      q = p+1;
    }
  if (p != q) { ++c; l = p - q; s += l + 1; }
  if (count_return) *count_return = c;
  if (!(text_list = (char **)malloc(sizeof(char *)*(c+1)))) return NULL;
  if (!(text_list[0] = (char *)malloc(sizeof(char)*(s+1)))) { free(text_list); return NULL; }
  memset(text_list[0], 0, s+1);
  s = 0;
  for (q=p=(char *)ptr; p!=(char *)ptr+size; ++p)
    if (*p == '\n') {
      l = p - q + 1;
      s += l + 1;
      memcpy(text_list[i], q, sizeof(char)*l);
      text_list[i+1] = text_list[i] + l + 1;
      ++i;
      q = p+1;
    }
  if (p != q) {
    l = p - q;
    memcpy(text_list[i], q, sizeof(char)*l);
    text_list[i+1] = text_list[i] + l + 1;
    ++i;
  }
  text_list[i] = NULL;
  return text_list;
#endif
}

#endif
