/*
  xclipview.cc - X cut buffer and X selection viewer

  Copyright (C) 2012 Taiji Yamada <taiji@aihara.co.jp>

  This is distributed under the MIT ("X") License.
 */
#if defined(HAVE_CONFIG_H)
#include "config.h"
#endif
#include <iostream>
#include <string>
#include <cstdlib>
#include "xcutbuffer.hpp"
#include "xselection.hpp"

bool output_mode = true;
int cutbuffer_index = -1;
char *selection_name = (char *)"PRIMARY";
int cutbuffer_main(int argc, char *argv[])
{
  if (output_mode) {
    xcutbuffer_getter<x11_text_view> v(argc, argv, 640, 480, cutbuffer_index);
    return v.loop();
  }
  xcutbuffer_setter<x11_text_view> v(argc, argv, 640, 480, cutbuffer_index);
  return v.loop();
}
int selection_main(int argc, char *argv[])
{
  if (output_mode) {
    xselection_getter<xt_text_view> v(argc, argv, 640, 480, selection_name);
    return v.loop();
  }
  xselection_setter<xt_text_view> v(argc, argv, 640, 480, selection_name);
  return v.loop();
}
int usage(std::ostream &out, const std::string me)
{
  out << "usage: " << me << " [-v] [-c buffer_index|-s PRIMARY|SECONDARY|CLIPBOARD] [-i < file]" << std::endl;
  return (out == std::cout) ? 0 : 1;
}
int main(int argc, char *argv[])
{
  int a;
  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cout, argv[0]);
    else if (std::string(argv[a]) == "-v")
      verbose = !0;
    else if (std::string(argv[a]) == "-i")
      output_mode = false;
    else if (std::string(argv[a]) == "-c") {
      cutbuffer_index = 0;
      if (a+1 < argc && (std::isdigit(argv[a+1][0]) ||
                         (argv[a+1][0] == '-' && std::isdigit(argv[a+1][1]))))
        cutbuffer_index = atoi(argv[++a]);
    }
    else if (std::string(argv[a]) == "-s") {
      if (a+1 < argc)
        selection_name = argv[++a];
    }

  if (cutbuffer_index != -1)
    return cutbuffer_main(argc, argv);
  return selection_main(argc, argv);
}
