/*
  xselection.cc - X selection command line interface

  Copyright (C) 2012 Taiji Yamada <taiji@aihara.co.jp>

  This is distributed under the MIT ("X") License.
 */
#if defined(HAVE_CONFIG_H)
#include "config.h"
#endif
#include <iostream>
#include <string>
#include <cstdlib>
#include <cctype>
#include "xselection.hpp"

bool output_mode = false, input_merge = false, exchange_mode = false;
bool input_continue = false, input_follow = false;
char *selection_name = (char *)"PRIMARY";
int rotation_number = 0;
int usage(std::ostream &out, const std::string me)
{
  out << "usage: " << me << " [-v] [-s PRIMARY|SECONDARY|CLIPBOARD] [[-x|-r [rotation_number]]|-o|[-a] [-f|-t] < file]" << std::endl;
  return (out == std::cout) ? 0 : 1;
}
int main(int argc, char *argv[])
{
  int a;
  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cout, argv[0]);
    else if (std::string(argv[a]) == "-v")
      verbose = !0;
    else if (std::string(argv[a]) == "-o")
      output_mode = true;
    else if (std::string(argv[a]) == "-a")
      input_merge = true;
    else if (std::string(argv[a]) == "-f")
      input_continue = true;
    else if (std::string(argv[a]) == "-t")
      input_follow = true;
    else if (std::string(argv[a]) == "-s") {
      if (a+1 < argc)
        selection_name = argv[++a];
    }
    else if (std::string(argv[a]) == "-r") {
      rotation_number = 1;
      if (a+1 < argc && (std::isdigit(argv[a+1][0]) ||
                         (argv[a+1][0] == '-' && std::isdigit(argv[a+1][1]))))
        rotation_number = atoi(argv[++a]);
    }
    else if (std::string(argv[a]) == "-x")
      exchange_mode = true;

  if (exchange_mode) {
    xselection_swapper<xt_view> v(argc, argv);
    return v.loop();
  }
  else if (rotation_number) {
    xselection_rotator<xt_view> v(argc, argv, rotation_number);
    return v.loop();
  }
  if (output_mode) {
    xselection_getter<xt_view> v(argc, argv, selection_name);
    return v.loop();
  }
  if (input_follow) {
    if (input_merge) {
      xselection_following_adder<xt_view> v(argc, argv, selection_name);
      return v.loop();
    }
    xselection_following_setter<xt_view> v(argc, argv, selection_name);
    return v.loop();
  }
  if (input_continue) {
    if (input_merge) {
      xselection_continuing_adder<xt_view> v(argc, argv, selection_name);
      return v.loop();
    }
    xselection_continuing_setter<xt_view> v(argc, argv, selection_name);
    return v.loop();
  }
  if (input_merge) {
    xselection_adder<xt_view> v(argc, argv, selection_name);
    return v.loop();
  }
  xselection_setter<xt_view> v(argc, argv, selection_name);
  return v.loop();
}
